package jello;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Scanner;
import java.io.File;
import java.io.FileNotFoundException;

public class Program {
    private static String help = "Useage: jello [flags] [options] file_name\nFlags:\n  -v: Displays version\n  -h: Displays this help section\nOptions:\n  -dbl: Sets the debug level, can be 0 to 2\n  -t: Sets internal clock rate in hertz";
    private static String versionNumber = "0.7.0";
    private static String versionName = "Jellal"; //Because everything is Jellal
    private static String version = "JELLO v" + versionNumber + " (" + versionName + ") by Jacob Allen";
    private static String[] validFlags = { "-v", "-h", "-jellal" };

	public static void main(String[] args) {
        int printLevel = 0;
        float tickRate = 0;
        
        if (args.length < 1)
        {
            System.out.println("Command requires arguments.");
            System.exit(1);
        }
        
        HashMap<String, String> options = ProcessArgs(args);

        if (options.containsKey("-jellal"))
        {
            System.out.println(version);
            System.out.println("Becuase everything is Jellal.");
            System.exit(0);
        }

        if (options.containsKey("-h"))
        {
            System.out.println(version);
            System.out.println(help);
            System.exit(0);
        }

        if (options.containsKey("-v"))
        {
            System.out.println(version);
            System.exit(0);
        }

        if (options.containsKey("-dbl"))
        {
            int dbl = 0;
            try
            {
            	dbl = Integer.parseUnsignedInt(options.get("-dbl"));
            }
            catch (NumberFormatException e)
            {
                System.out.println("Debug level must be an integer.");
                System.exit(1);
            }
            printLevel = dbl;
        }

        if (options.containsKey("-t"))
        {
            float tick = 0;
            try{
            	tick = Float.parseFloat(options.get("-t"));
            }
            catch(NumberFormatException e)
            {
                System.out.println("Clock speed must be in hertz.");
                System.exit(1);
            }
            tickRate = tick;
        }

        if (!options.containsKey("*filename*"))
        {
            System.out.println("A file name is required.");
            System.exit(1);
        }

        System.out.println(version);

        System.out.print("Reading File... ");

        List<String> codelines = LoadFile(options.get("*filename*"));

        System.out.println("Done.");

        System.out.println("Processing Commands...");

        RunCode(codelines, tickRate, printLevel);

        System.out.println("Done.");
	}
	
	private static HashMap<String, String> ProcessArgs(String[] args)
    {
        List<String> toProcess = Arrays.asList(args);
        HashMap<String, String> options = new HashMap<String, String>();
        int i = 0;
        while (toProcess.size() > i)
        {
            String item = toProcess.get(i);
            i++;
            if (item.charAt(0) == '-')
            {
                if (Arrays.asList(validFlags).contains(item))
                {
                    options.put(item, "true");
                }
                else
                {
                    if (toProcess.size() > 0)
                    {
                    	options.put(item, toProcess.get(i));
                    	i++;
                    }
                    else
                    {
                    	System.out.println("Option requires value.");
                        System.exit(1);
                    }
                }

            }
            else
            {
                if (options.containsKey("*filename*"))
                {
                	System.out.println("Only one file name is required.");
                	System.exit(1);
                }
                else
                {
                    options.put("*filename*", item);
                }
            }
        }

        return options;
    }
	
    private static List<String> LoadFile(String filename)
    {
        List<String> codelines = new ArrayList<String>();
        try
        {
            Scanner file = new Scanner(new File(filename));
            while (file.hasNextLine())
            {
                codelines.add(file.nextLine());
            }
            file.close();
        }
        catch (FileNotFoundException e)
        {
            System.out.println("Error file '" + filename + "' not found.");
            System.exit(1);
        }
        return codelines;
    }
    
    private static boolean RunCode(List<String> codelines, float tickRate, int debugLevel)
    {
        InternalState.SetPointer(0);
        InternalState.SetCodeLines(codelines);
        int previousPointer = 0;

        while (InternalState.GetPointer() < InternalState.GetCodeLines().size())
        {
            String line = InternalState.GetCodeLine(InternalState.GetPointer());
            
            if (line.length() < 1)
            {
            	InternalState.SetPointer(InternalState.GetPointer() + 1);
                continue;
            }
            
            if (line.charAt(0) == '#' || line.charAt(0) == ':')
            {
            	InternalState.SetPointer(InternalState.GetPointer() + 1);
                continue;
            }

            List<String> parts = new ArrayList<String>(Arrays.asList(line.split("\\s+")));
            String command = parts.remove(0);
            String[] values = parts.toArray(new String[] {});

            if (debugLevel > 0)
                System.out.println("  -> " + line + " AT " + (InternalState.GetPointer() + 1));

            previousPointer = InternalState.GetPointer();

            int pointerMove = Commands.RunCommand(command, values);

            InternalState.SetPointer(InternalState.GetPointer() + pointerMove);

            if (InternalState.CleanUp())
            {
            	System.out.println("Error on line " + (previousPointer+1) + " at " + codelines.get(previousPointer));
            	System.out.println(InternalState.Dump());
            	System.out.println("Internal state error, program terminating.");
                System.exit(1);
            }

            if (debugLevel > 1)
            	System.out.println(InternalState.Dump());

            if (tickRate > 0)
            {
				try {
					Thread.sleep((long)((1.0f / tickRate) * 1000));
				} catch (InterruptedException e) {
		            System.out.println("Timing interrupted.");
		            System.exit(1);
				}
            }
        }
        return true;
    }
}
